#!/bin/bash

# Check if FPGA version is correct

target_version="3.6.17"
fpga_version=$(SPEC_KTSAL_EN=true HW_GetFpgaInfo 0 | head -n 1 | cut -d" " -f2)
update_status="/srv/www/xml-status/update_status.xml"

if [[ ${fpga_version} -lt 132 ]] ; then
    echo "FPGA Version must be >1.32"
    exit 1
fi

# Loop through option cards
if [ -z $1 ] ; then
    echo "Needs 1 argument"
    exit 1
fi

bundle_name="$(basename $1)"
bundle_dir="$(dirname $1)"
bundle_md5="$(md5sum ${bundle_dir}/${bundle_name} | cut -d" " -f1)"

cat > ${bundle_dir}/install.sh << EOF
#!/bin/sh -x
echo "Bundle md5sum: \$(md5sum ${bundle_name} | cut -d" " -f1)"
echo "Target md5sum: ${bundle_md5}"

if [ "\$(md5sum ${bundle_name} | cut -d" " -f1)" != "${bundle_md5}" ]; then
    echo "Transfer failed: evk bundle md5 mismatch"
    exit 1
fi
if ! tar -xvzf ${bundle_name} ; then
    echo "Upgrade failed: unable to untar evk bundle"
fi

if ./${bundle_name:0:-7}/install.sh ; then
    echo "Upgrade completed"
    reboot
else
    echo "Upgrade failed: installation unsuccessful"
fi
EOF

slot_addr=(00 08 10 18 20 28 30)
for slot in {1..6}; do
    dcs_slot=$(SPEC_KTSAL_EN=true DCS_GetCardInfo 0 ${slot} | head -n 1)
    card_id=$(echo "${dcs_slot}" | cut -d" " -f 1)
    stl_fpga=$(echo "${dcs_slot}" | cut -d" " -f 4)

    echo "Slot: $slot"
    echo "cardid: ${card_id}"
    echo "FPGA: ${stl_fpga}"
    if [[ ${card_id} -ne 62 ]] || [[ ${stl_fpga} -lt 258 ]]; then
        continue
    fi
    echo "Found upgradeable STL card in slot:${slot}"

    stl_inst=$(SPEC_KTSAL_EN=true DCS_GetInstance 0 ${slot} 73 0 | grep -i "System Feature Instance" | cut -d":" -f2 | tr -d ' ')
    stl_info=$(SPEC_KTSAL_EN=true STR_GetVersion 0 ${stl_inst} | head -n 1)
    stl_version=$(echo "${stl_info}" | cut -d" " -f1)
    stl_serial=$(echo "${stl_info}" | cut -d" " -f3)

    upgrade_seq=$(wc -l ${update_status} | cut -d" " -f1)

    if [[ "${stl_version}" == "${target_version}" ]] ; then
        upgrade_str="<sequence step=\"${upgrade_seq}\" stepName=\"STL${stl_inst}\" d1=\"${stl_version}\" d2=\"${target_version}\" d3=\"No Upgrade Needed\" status=\"Completed\"/>"
        sed -i "\#</LogEvent#i ${upgrade_str}" ${update_status}
        echo "STL card in slot ${slot} is already at version ${target_version}"
        continue
    fi

    echo "Upgrading STL card in slot ${slot} to ${target_version}"
    upgrade_str="<sequence step=\"${upgrade_seq}\" stepName=\"STL${stl_inst}\" d1=\"${stl_version}\" d2=\"${target_version}\" d3=\"Upgrade Needed\" status=\"In Progress\"/>"
    sed -i "\#</LogEvent#i ${upgrade_str}" ${update_status}

    device="/dev/ttyS3"
    io_port=$(lspci -v -d 1ad7:* | grep "I/O ports at " | cut -d" " -f4)
    io_port="0x${io_port:0:(-2)}${slot_addr[${slot}]}"

    setserial ${device} port ${io_port} irq 0 uart 16950 Baud_base 3125000
    setserial -g ${device}

    echo "Running kermit script"
    echo "stl_update.ck $(dirname $1) $(basename $1) ${stl_serial}"
    if stl_update.ck ${bundle_dir} ${bundle_name} ${stl_serial} ; then
        upgrade_str2=$(echo ${upgrade_str} | sed "s/Upgrade Needed/Upgraded/")
        upgrade_str2=$(echo ${upgrade_str2} | sed "s/In Progress/Completed/")
        sed -i "s#${upgrade_str}#${upgrade_str2}#" ${update_status}
        echo "Upgrade Passed"
    else
        upgrade_str2=$(echo ${upgrade_str} | sed "s/Upgrade Needed/Error/")
        upgrade_str2=$(echo ${upgrade_str2} | sed "s/In Progress/Failed/")
        sed -i "s#${upgrade_str}#${upgrade_str2}#" ${update_status}
        echo "Upgrade Failed"
    fi
done

